#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

'''
These are tags that halo custom edition will magically seek out at
runtime by their name. Not every map needs to have these, but depending
on which ones are missing, different things can happen.
Maps might not load, menus might not appear, sounds might not play, etc.
'''

# unicode_string_list
HARDCODED_ustr_TAG_PATHS = (
    "ui\\multiplayer_game_text.unicode_string_list",
    "ui\\random_player_names.unicode_string_list",

    "ui\\shell\\main_menu\\credits_screen.unicode_string_list",
    "ui\\shell\\main_menu\\map_list_short.unicode_string_list",
    "ui\\shell\\main_menu\\map_list_oneline.unicode_string_list",
    "ui\\shell\\main_menu\\mp_map_list.unicode_string_list",

    "ui\\shell\\main_menu\\multiplayer_type_select\\join_game\\join_game_rules_strings.unicode_string_list",
    "ui\\shell\\main_menu\\multiplayer_type_select\\join_game\\join_game_ticker_labels.unicode_string_list",

    "ui\\shell\\main_menu\\player_profiles_select\\button_set_long_descriptions.unicode_string_list",
    "ui\\shell\\main_menu\\player_profiles_select\\button_set_short_descriptions.unicode_string_list",# XBOX_ONLY
    "ui\\shell\\main_menu\\player_profiles_select\\joystick_set_defaults_descriptions.unicode_string_list",
    "ui\\shell\\main_menu\\player_profiles_select\\joystick_set_short_descriptions.unicode_string_list",
    "ui\\shell\\main_menu\\player_profiles_select\\profile_description_labels.unicode_string_list",

    "ui\\shell\\main_menu\\settings_select\\player_setup\\player_profile_edit\\color_edit\\colors_list.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\player_setup\\player_profile_edit\\controls_setup\\controls_axis_direction_names.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\player_setup\\player_profile_edit\\controls_setup\\controls_device_labels.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\player_setup\\player_profile_edit\\controls_setup\\controls_gamepad_names.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\player_setup\\player_profile_edit\\controls_setup\\controls_keyboard_button_names.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\player_setup\\player_profile_edit\\controls_setup\\controls_mouse_button_names.unicode_string_list",

    "ui\\shell\\main_menu\\settings_select\\multiplayer_setup\\item_options_edit\\var_weapon_set.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\multiplayer_setup\\player_options_edit\\player_options_labels.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\multiplayer_setup\\player_options_edit\\var_invisible_players.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\multiplayer_setup\\player_options_edit\\var_maximum_health.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\multiplayer_setup\\player_options_edit\\var_number_of_lives.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\multiplayer_setup\\player_options_edit\\var_odd_man_out.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\multiplayer_setup\\player_options_edit\\var_respawn_time.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\multiplayer_setup\\player_options_edit\\var_respawn_time_growh.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\multiplayer_setup\\player_options_edit\\var_shields.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\multiplayer_setup\\player_options_edit\\var_suicide_penalty.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\multiplayer_setup\\playlist_edit\\ctf_edit\\ctf_labels.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\multiplayer_setup\\playlist_edit\\ctf_edit\\var_assault.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\multiplayer_setup\\playlist_edit\\ctf_edit\\var_captures_to_win.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\multiplayer_setup\\playlist_edit\\ctf_edit\\var_flag_at_home.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\multiplayer_setup\\playlist_edit\\ctf_edit\\var_flag_must_reset.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\multiplayer_setup\\playlist_edit\\ctf_edit\\var_single_flag.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\multiplayer_setup\\playlist_edit\\ctf_edit\\var_time_limit.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\multiplayer_setup\\playlist_edit\\koth_edit\\koth_labels.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\multiplayer_setup\\playlist_edit\\koth_edit\\var_moving_hill.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\multiplayer_setup\\playlist_edit\\koth_edit\\var_score_to_win.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\multiplayer_setup\\playlist_edit\\oddball_edit\\oddball_labels.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\multiplayer_setup\\playlist_edit\\oddball_edit\\var_ball_spawn_count.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\multiplayer_setup\\playlist_edit\\oddball_edit\\var_ball_type.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\multiplayer_setup\\playlist_edit\\oddball_edit\\var_random_start.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\multiplayer_setup\\playlist_edit\\oddball_edit\\var_score_to_win.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\multiplayer_setup\\playlist_edit\\oddball_edit\\var_speed_with_ball.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\multiplayer_setup\\playlist_edit\\oddball_edit\\var_trait_with_ball.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\multiplayer_setup\\playlist_edit\\race_edit\\race_labels.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\multiplayer_setup\\playlist_edit\\race_edit\\var_laps_to_win.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\multiplayer_setup\\playlist_edit\\race_edit\\var_race_type.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\multiplayer_setup\\playlist_edit\\race_edit\\var_team_play.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\multiplayer_setup\\playlist_edit\\race_edit\\var_team_scoring.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\multiplayer_setup\\playlist_edit\\slayer_edit\\slayer_labels.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\multiplayer_setup\\playlist_edit\\slayer_edit\\var_death_bonus.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\multiplayer_setup\\playlist_edit\\slayer_edit\\var_kill_in_order.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\multiplayer_setup\\playlist_edit\\slayer_edit\\var_kill_penalty.unicode_string_list",# UNUSED?
    "ui\\shell\\main_menu\\settings_select\\multiplayer_setup\\playlist_edit\\slayer_edit\\var_kills_to_win.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\multiplayer_setup\\playlist_edit\\slayer_edit\\var_team_play.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\multiplayer_setup\\teamplay_options_edit\\teamplay_options_labels.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\multiplayer_setup\\teamplay_options_edit\\var_friendly_fire.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\multiplayer_setup\\teamplay_options_edit\\var_friendly_fire_penalty.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\multiplayer_setup\\vehicle_options_edit\\var_vehicle_set.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\multiplayer_setup\\vehicle_options_edit\\var_vehicles_respawn.unicode_string_list",
    "ui\\shell\\main_menu\\settings_select\\multiplayer_setup\\vehicle_options_edit\\vehicle_options_labels.unicode_string_list",

    "ui\\shell\\strings\\common_button_captions.unicode_string_list",
    "ui\\shell\\strings\\default_player_profile_names.unicode_string_list",
    "ui\\shell\\strings\\game_variant_descriptions.unicode_string_list",
    "ui\\shell\\strings\\loading.unicode_string_list",
    "ui\\shell\\strings\\temp_strings.unicode_string_list",
    )

# ui_widget_definition
HARDCODED_DeLa_TAG_PATHS = (
    "ui\\shell\\main_menu\\main_menu.ui_widget_definition",
    "ui\\shell\\main_menu\\multiplayer_type_select\\connected\\pregame\\connected_pregame_screen.ui_widget_definition",# XBOX_ONLY

    "ui\\shell\\multiplayer_game\\pause_game\\1p_pause_game.ui_widget_definition",
    "ui\\shell\\multiplayer_game\\pause_game\\2p_pause_game.ui_widget_definition",
    "ui\\shell\\multiplayer_game\\pause_game\\4p_pause_game.ui_widget_definition",

    "ui\\shell\\solo_game\\pause_game\\pause_game_split_screen.ui_widget_definition",
    "ui\\shell\\solo_game\\pause_game\\pause_game.ui_widget_definition",

    "ui\\shell\\solo_game\\player_help\\player_help_screen_a10.ui_widget_definition",
    "ui\\shell\\solo_game\\player_help\\player_help_screen_a30.ui_widget_definition",
    "ui\\shell\\solo_game\\player_help\\player_help_screen_a50.ui_widget_definition",
    "ui\\shell\\solo_game\\player_help\\player_help_screen_b30.ui_widget_definition",
    "ui\\shell\\solo_game\\player_help\\player_help_screen_b40.ui_widget_definition",
    "ui\\shell\\solo_game\\player_help\\player_help_screen_c10.ui_widget_definition",
    "ui\\shell\\solo_game\\player_help\\player_help_screen_c20.ui_widget_definition",
    "ui\\shell\\solo_game\\player_help\\player_help_screen_c40.ui_widget_definition",
    "ui\\shell\\solo_game\\player_help\\player_help_screen_d20.ui_widget_definition",
    "ui\\shell\\solo_game\\player_help\\player_help_screen_d40.ui_widget_definition",

    "ui\\shell\\error\\error_modal_fullscreen.ui_widget_definition",
    "ui\\shell\\error\\error_modal_halfscreen.ui_widget_definition",
    "ui\\shell\\error\\error_modal_qtrscreen.ui_widget_definition"
    "ui\\shell\\error\\error_nonmodal_fullscreen.ui_widget_definition",
    "ui\\shell\\error\\error_nonmodal_halfscreen.ui_widget_definition",
    "ui\\shell\\error\\error_nonmodal_qtrscreen.ui_widget_definition",
    )

# font
HARDCODED_font_TAG_PATHS = (
    "ui\\gamespy.font",
    "ui\\large_ui.font",
    "ui\\small_ui.font",
    )

# virtual_keyboard
HARDCODED_vcky_TAG_PATHS = (
    "ui\\english.virtual_keyboard",
    )

# bitmap
HARDCODED_bitm_TAG_PATHS = (
    "ui\\shell\\bitmaps\\cursor.bitmap",
    "ui\\shell\\bitmaps\\background.bitmap",
    "ui\\shell\\bitmaps\\team_background.bitmap",
    "ui\\shell\\bitmaps\\team_icon_slayer.bitmap",
    "ui\\shell\\bitmaps\\team_icon_race.bitmap",
    "ui\\shell\\bitmaps\\team_icon_oddball.bitmap",
    "ui\\shell\\bitmaps\\team_icon_king.bitmap",
    "ui\\shell\\bitmaps\\team_icon_ctf.bitmap",
    "ui\\shell\\bitmaps\\trouble_brewing.bitmap",
    "ui\\shell\\bitmaps\\white.bitmap",
    )


# globals
HARDCODED_matg_TAG_PATHS = (
    "globals\\globals.globals",
    )


# scenario
HARDCODED_scnr_TAG_PATHS = (
    "levels\\ui\\ui.scenario",
    )


# sound_looping
HARDCODED_lsnd_TAG_PATHS = (
    "sound\\music\\title1\\title1.sound_looping",
    )


# sound
HARDCODED_snd__TAG_PATHS = (
    "sound\\sfx\\ui\\flag_failure.sound",
    "sound\\sfx\\ui\\back.sound",
    "sound\\sfx\\ui\\forward.sound",
    "sound\\sfx\\ui\\cursor.sound",
    )

HARDCODED_TAG_PATHS = frozenset(
    HARDCODED_ustr_TAG_PATHS + HARDCODED_DeLa_TAG_PATHS +
    HARDCODED_font_TAG_PATHS + HARDCODED_vcky_TAG_PATHS +
    HARDCODED_bitm_TAG_PATHS +
    HARDCODED_matg_TAG_PATHS + HARDCODED_scnr_TAG_PATHS +
    HARDCODED_lsnd_TAG_PATHS + HARDCODED_snd__TAG_PATHS
    )



#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

__all__ = ( 'JmsVertex', )


class JmsVertex:
    __slots__ = (
        "node_0",
        "pos_x", "pos_y", "pos_z",
        "norm_i", "norm_j", "norm_k",
        "binorm_i", "binorm_j", "binorm_k",
        "tangent_i", "tangent_j", "tangent_k",
        "node_1", "node_1_weight",
        "tex_u", "tex_v", "tex_w",
        "other_nodes", "other_weights", "other_uvws"
        )
    def __init__(self, node_0=0,
                 pos_x=0.0, pos_y=0.0, pos_z=0.0,
                 norm_i=0.0, norm_j=0.0, norm_k=1.0,
                 node_1=-1, node_1_weight=0.0,
                 tex_u=0, tex_v=0, tex_w=0,
                 binorm_i=0.0,  binorm_j=1.0,  binorm_k=0.0,
                 tangent_i=1.0, tangent_j=0.0, tangent_k=0.0,
                 other_nodes=(), other_weights=(), other_uvws=()):
        if node_1_weight <= 0:
            node_1 = -1
            node_1_weight = 0

        self.node_0 = node_0
        self.pos_x = pos_x
        self.pos_y = pos_y
        self.pos_z = pos_z
        self.norm_i = norm_i
        self.norm_j = norm_j
        self.norm_k = norm_k
        self.binorm_i = binorm_i
        self.binorm_j = binorm_j
        self.binorm_k = binorm_k
        self.tangent_i = tangent_i
        self.tangent_j = tangent_j
        self.tangent_k = tangent_k
        self.node_1 = node_1
        self.node_1_weight = node_1_weight
        self.tex_u = tex_u
        self.tex_v = tex_v
        self.tex_w = tex_w
        self.other_nodes = other_nodes
        self.other_weights = other_weights
        self.other_uvws = other_uvws

    def __repr__(self):
        return """JmsVertex(node_0=%s,
    x=%s, y=%s, z=%s,
    i=%s, j=%s, k=%s,
    node_1=%s, node_1_weight=%s,
    u=%s, v=%s, w=%s
)""" % (self.node_0,
        self.pos_x, self.pos_y, self.pos_z,
        self.norm_i, self.norm_j, self.norm_k,
        self.node_1, self.node_1_weight,
        self.tex_u, self.tex_v, self.tex_w)

    def __eq__(self, other):
        if not isinstance(other, JmsVertex):
            return False
        elif (abs(self.pos_z  - other.pos_z)  > 0.00001 or
              abs(self.norm_k - other.norm_k) > 0.0001):
            return False
        elif (abs(self.pos_x - other.pos_x) > 0.00001 or
              abs(self.pos_y - other.pos_y) > 0.00001):
            return False
        elif (abs(self.norm_i - other.norm_i) > 0.0001 or
              abs(self.norm_j - other.norm_j) > 0.0001):
            return False
        elif abs(self.node_1_weight - other.node_1_weight) > 0.0001:
            return False
        elif self.node_0 != other.node_0 or self.node_1 != other.node_1:
            return False

        return (abs(self.tex_u - other.tex_u) <= 0.0001 and
                abs(self.tex_v - other.tex_v) <= 0.0001)



#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

############# Credits and version info #############
# Definition generated from Assembly XML tag def
#	 Date generated: 2018/12/03  04:56
#
# revision: 1		author: Assembly
# 	Generated plugin from scratch.
# revision: 2		author: shade45
# 	Named some values
# revision: 3		author: -DeToX-
# 	Named some reflexives and idents
# revision: 4		author: Veegie
# 	Mapped out reflextives and idents. Fixed titles.
# revision: 5		author: Lord Zedd
# 	Updates and standardizing
# revision: 6		author: Moses_of_Egypt
# 	Cleaned up and converted to SuPyr definition
#
####################################################

from ..common_descs import *
from .objs.tag import *
from supyr_struct.defs.tag_def import TagDef


matg_havok_cleanup_resource = Struct("havok_cleanup_resource",
    h3_dependency("object_cleanup_effect"),
    ENDIAN=">", SIZE=16
    )


matg_sound_global = Struct("sound_global",
    h3_dependency("sound_classes"),
    h3_dependency("sound_effects"),
    h3_dependency("sound_mix"),
    h3_dependency("sound_combat_dialogue_constants"),
    h3_dependency("sound_global_propagation"),
    ENDIAN=">", SIZE=80
    )


matg_ai_global_gravemind_propertie = QStruct("gravemind_propertie",
    Float("minimum_retreat_time"),
    Float("ideal_retreat_time"),
    Float("maximum_retreat_time"),
    ENDIAN=">", SIZE=12
    )


matg_ai_global_style = Struct("style",
    h3_dependency("style"),
    ENDIAN=">", SIZE=16
    )


matg_ai_global = Struct("ai_global",
    BytesRaw("unknown_0", SIZE=12, VISIBLE=False),
    Float("danger_broadly_facing"),
    BytesRaw("unknown_1", SIZE=4, VISIBLE=False),
    Float("danger_shooting_near"),
    BytesRaw("unknown_2", SIZE=4, VISIBLE=False),
    Float("danger_shooting_at"),
    BytesRaw("unknown_3", SIZE=4, VISIBLE=False),
    Float("danger_extremely_close"),
    BytesRaw("unknown_4", SIZE=4, VISIBLE=False),
    Float("danger_shield_damage"),
    Float("danger_extended_shield_damage"),
    Float("danger_body_damage"),
    Float("danger_extended_body_damage"),
    BytesRaw("unknown_5", SIZE=48, VISIBLE=False),
    h3_dependency("global_dialogue"),
    h3_string_id("default_mission_dialogue_sound_effect"),
    BytesRaw("unknown_6", SIZE=20, VISIBLE=False),
    Float("jump_down"),
    Float("jump_step"),
    Float("jump_crouch"),
    Float("jump_stand"),
    Float("jump_storey"),
    Float("jump_tower"),
    Float("max_jump_down_height_down"),
    Float("max_jump_down_height_step"),
    Float("max_jump_down_height_crouch"),
    Float("max_jump_down_height_stand"),
    Float("max_jump_down_height_storey"),
    Float("max_jump_down_height_tower"),
    QStruct("hoist_step", INCLUDE=from_to),
    QStruct("hoist_crouch", INCLUDE=from_to),
    QStruct("hoist_stand", INCLUDE=from_to),
    BytesRaw("unknown_7", SIZE=24, VISIBLE=False),
    QStruct("vault_step", INCLUDE=from_to),
    QStruct("vault_crouch", INCLUDE=from_to),
    BytesRaw("unknown_8", SIZE=48, VISIBLE=False),
    h3_reflexive("gravemind_properties", matg_ai_global_gravemind_propertie),
    BytesRaw("unknown_9", SIZE=48, VISIBLE=False),
    Float("scary_target_threshold"),
    Float("scary_weapon_threshold"),
    BytesRaw("unknown_10", SIZE=44, VISIBLE=False),
    h3_reflexive("styles", matg_ai_global_style),
    ENDIAN=">", SIZE=432
    )


matg_damage_table_damage_group_armor_modifier = Struct("armor_modifier",
    h3_string_id("name"),
    Float("damage_multiplier"),
    ENDIAN=">", SIZE=8
    )


matg_damage_table_damage_group = Struct("damage_group",
    h3_string_id("name"),
    h3_reflexive("armor_modifiers", matg_damage_table_damage_group_armor_modifier),
    ENDIAN=">", SIZE=16
    )


matg_damage_table = Struct("damage_table",
    h3_reflexive("damage_groups", matg_damage_table_damage_group),
    ENDIAN=">", SIZE=12
    )


matg_sound = Struct("sound",
    h3_dependency("sound_obsolete"),
    ENDIAN=">", SIZE=16
    )


matg_camera = Struct("camera",
    h3_dependency("default_unit_camera_track"),
    Array("unknown_array_0", SUB_STRUCT=Float("unknown"), SIZE=24, VISIBLE=False),
    SInt16("unknown_0", VISIBLE=False),
    SInt16("unknown_1", VISIBLE=False),
    Array("unknown_array_1", SUB_STRUCT=Float("unknown"), SIZE=11, VISIBLE=False),
    ENDIAN=">", SIZE=160
    )


matg_player_control_look_function = QStruct("look_function",
    Float("scale"),
    VISIBLE=False,
    ENDIAN=">", SIZE=4
    )


matg_player_control = Struct("player_control",
    Float("magnetism_friction"),
    Float("magnetism_adhesion"),
    Float("inconsequential_target_scale"),
    BytesRaw("unknown_0", SIZE=12, VISIBLE=False),
    QStruct("crosshair_location", INCLUDE=xy_float),
    Float("seconds_to_start"),
    Float("seconds_to_full_speed"),
    Float("decay_rate"),
    Float("full_speed_multiplier"),
    Float("pegged_magnitude"),
    Float("pegged_angular_threshold"),
    BytesRaw("unknown_1", SIZE=8, VISIBLE=False),
    Float("look_default_pitch_rate"),
    Float("look_default_yaw_rate"),
    Float("look_peg_threshold"),
    Float("look_yaw_acceleration_time"),
    Float("look_yaw_acceleration_scale"),
    Float("look_pitch_acceleration_time"),
    Float("look_pitch_acceleration_scale"),
    Float("look_autoleveling_scale"),
    BytesRaw("unknown_2", SIZE=8, VISIBLE=False),
    Float("gravity_scale"),
    SInt32("minimum_autoleveling_ticks"),
    float_rad("minimum_angle_for_vehicle_flipping"),
    h3_reflexive("look_function", matg_player_control_look_function),
    Float("minimum_action_hold_time"),
    BytesRaw("unknown_3", SIZE=4, VISIBLE=False),
    ENDIAN=">", SIZE=136
    )


matg_difficulty = QStruct("difficulty",
    Float("easy_enemy_damage"),
    Float("normal_enemy_damage"),
    Float("hard_enemy_damage"),
    Float("impossible_enemy_damage"),
    Float("easy_enemy_vitality"),
    Float("normal_enemy_vitality"),
    Float("hard_enemy_vitality"),
    Float("impossible_enemy_vitality"),
    Float("easy_enemy_shield"),
    Float("normal_enemy_shield"),
    Float("hard_enemy_shield"),
    Float("impossible_enemy_shield"),
    Float("easy_enemy_recharge"),
    Float("normal_enemy_recharge"),
    Float("hard_enemy_recharge"),
    Float("impossible_enemy_recharge"),
    Float("easy_friend_damage"),
    Float("normal_friend_damage"),
    Float("hard_friend_damage"),
    Float("impossible_friend_damage"),
    Float("easy_friend_vitality"),
    Float("normal_friend_vitality"),
    Float("hard_friend_vitality"),
    Float("impossible_friend_vitality"),
    Float("easy_friend_shield"),
    Float("normal_friend_shield"),
    Float("hard_friend_shield"),
    Float("impossible_friend_shield"),
    Float("easy_friend_recharge"),
    Float("normal_friend_recharge"),
    Float("hard_friend_recharge"),
    Float("impossible_friend_recharge"),
    Float("easy_infection_forms"),
    Float("normal_infection_forms"),
    Float("hard_infection_forms"),
    Float("impossible_infection_forms"),
    Float("easy_unknown_0", VISIBLE=False),
    Float("normal_unknown_0", VISIBLE=False),
    Float("hard_unknown_0", VISIBLE=False),
    Float("impossible_unknown_0", VISIBLE=False),
    Float("easy_rate_of_fire"),
    Float("normal_rate_of_fire"),
    Float("hard_rate_of_fire"),
    Float("impossible_rate_of_fire"),
    Float("easy_projectile_error"),
    Float("normal_projectile_error"),
    Float("hard_projectile_error"),
    Float("impossible_projectile_error"),
    Float("easy_burst_error"),
    Float("normal_burst_error"),
    Float("hard_burst_error"),
    Float("impossible_burst_error"),
    Float("easy_target_delay"),
    Float("normal_target_delay"),
    Float("hard_target_delay"),
    Float("impossible_target_delay"),
    Float("easy_burst_separation"),
    Float("normal_burst_separation"),
    Float("hard_burst_separation"),
    Float("impossible_burst_separation"),
    Float("easy_target_tracking"),
    Float("normal_target_tracking"),
    Float("hard_target_tracking"),
    Float("impossible_target_tracking"),
    Float("easy_target_leading"),
    Float("normal_target_leading"),
    Float("hard_target_leading"),
    Float("impossible_target_leading"),
    Float("easy_overcharge_chance"),
    Float("normal_overcharge_chance"),
    Float("hard_overcharge_chance"),
    Float("impossible_overcharge_chance"),
    Float("easy_special_fire_delay"),
    Float("normal_special_fire_delay"),
    Float("hard_special_fire_delay"),
    Float("impossible_special_fire_delay"),
    Float("easy_guidance_vs_player"),
    Float("normal_guidance_vs_player"),
    Float("hard_guidance_vs_player"),
    Float("impossible_guidance_vs_player"),
    Float("easy_melee_delay_base"),
    Float("normal_melee_delay_base"),
    Float("hard_melee_delay_base"),
    Float("impossible_melee_delay_base"),
    Float("easy_melee_delay_scale"),
    Float("normal_melee_delay_scale"),
    Float("hard_melee_delay_scale"),
    Float("impossible_melee_delay_scale"),
    Float("easy_unknown_1", VISIBLE=False),
    Float("normal_unknown_1", VISIBLE=False),
    Float("hard_unknown_1", VISIBLE=False),
    Float("impossible_unknown_1", VISIBLE=False),
    Float("easy_grenade_chance_scale"),
    Float("normal_grenade_chance_scale"),
    Float("hard_grenade_chance_scale"),
    Float("impossible_grenade_chance_scale"),
    Float("easy_grenade_timer_scale"),
    Float("normal_grenade_timer_scale"),
    Float("hard_grenade_timer_scale"),
    Float("impossible_grenade_timer_scale"),
    Float("easy_unknown_2", VISIBLE=False),
    Float("normal_unknown_2", VISIBLE=False),
    Float("hard_unknown_2", VISIBLE=False),
    Float("impossible_unknown_2", VISIBLE=False),
    Float("easy_unknown_3", VISIBLE=False),
    Float("normal_unknown_3", VISIBLE=False),
    Float("hard_unknown_3", VISIBLE=False),
    Float("impossible_unknown_3", VISIBLE=False),
    Float("easy_unknown_4", VISIBLE=False),
    Float("normal_unknown_4", VISIBLE=False),
    Float("hard_unknown_4", VISIBLE=False),
    Float("impossible_unknown_4", VISIBLE=False),
    Float("easy_major_upgrade_normal"),
    Float("normal_major_upgrade_normal"),
    Float("hard_major_upgrade_normal"),
    Float("impossible_major_upgrade_normal"),
    Float("easy_major_upgrade_few"),
    Float("normal_major_upgrade_few"),
    Float("hard_major_upgrade_few"),
    Float("impossible_major_upgrade_few"),
    Float("easy_major_upgrade_many"),
    Float("normal_major_upgrade_many"),
    Float("hard_major_upgrade_many"),
    Float("impossible_major_upgrade_many"),
    Float("easy_player_vehicle_ram_chance"),
    Float("normal_player_vehicle_ram_chance"),
    Float("hard_player_vehicle_ram_chance"),
    Float("impossible_player_vehicle_ram_chance"),
    Pad(132),
    ENDIAN=">", SIZE=644
    )


matg_grenade = Struct("grenade",
    SInt16("maximum_count"),
    SInt16("unknown_0", VISIBLE=False),
    h3_dependency("throwing_effect"),
    BytesRaw("unknown_1", SIZE=16, VISIBLE=False),
    h3_dependency("equipment"),
    h3_dependency("projectile"),
    ENDIAN=">", SIZE=68
    )


matg_interface_tag = Struct("interface_tag",
    h3_dependency("spinner"),
    h3_dependency("obsolete"),
    h3_dependency("screen_color_table"),
    h3_dependency("hud_color_table"),
    h3_dependency("editor_color_table"),
    h3_dependency("dialog_color_table"),
    h3_dependency("motion_sensor_sweep_bitmap"),
    h3_dependency("motion_sensor_sweep_bitmap_mask"),
    h3_dependency("multiplayer_hud_bitmap"),
    h3_dependency("hud_digits_definition"),
    h3_dependency("motion_sensor_blip_bitmap"),
    h3_dependency("interface_goo_map_1"),
    h3_dependency("interface_goo_map_2"),
    h3_dependency("interface_goo_map_3"),
    h3_dependency("main_menu_ui_globals"),
    h3_dependency("single_player_ui_globals"),
    h3_dependency("multiplayer_ui_globals"),
    h3_dependency("hud_globals"),
    ENDIAN=">", SIZE=288
    )


matg_player_information = Struct("player_information",
    Float("walking_speed"),
    Float("run_forward"),
    Float("run_backward"),
    Float("run_sideways"),
    Float("run_acceleration", VISIBLE=False),
    Float("sneak_forward"),
    Float("sneak_backward"),
    Float("sneak_sideways"),
    Float("sneak_acceleration"),
    Float("airborne_acceleration"),
    QStruct("grenade_origin", INCLUDE=xyz_float),
    Float("stun_movement_penalty"),
    Float("stun_turning_penalty"),
    Float("stun_jumping_penalty"),
    QStruct("stun_time", INCLUDE=from_to),
    QStruct("first_person_idle_time", INCLUDE=from_to),
    Float("first_person_skip_fraction"),
    h3_dependency("unknown_0", VISIBLE=False),
    h3_dependency("unknown_1", VISIBLE=False),
    h3_dependency("coop_respawn_effect"),
    SInt32("binoculars_zoom_count"),
    QStruct("binocular_zoom_range", INCLUDE=from_to),
    h3_dependency("flashlight_on"),
    h3_dependency("flashlight_off"),
    h3_dependency("default_damage_response"),
    ENDIAN=">", SIZE=192
    )


matg_player_representation = Struct("player_representation",
    h3_dependency("first_person_hands"),
    h3_dependency("first_person_body"),
    h3_dependency("third_person_unit"),
    h3_string_id("third_person_variant"),
    h3_dependency("binoculars_zoom_in_sound"),
    h3_dependency("binoculars_zoom_out_sound"),
    ENDIAN=">", SIZE=84
    )


matg_falling_damage = Struct("falling_damage",
    QStruct("harmful_falling_distance", INCLUDE=from_to),
    h3_dependency("falling_damage"),
    h3_dependency("unknown_0", VISIBLE=False),
    h3_dependency("soft_landing"),
    h3_dependency("hard_landing"),
    h3_dependency("script_damage"),
    Float("maximum_falling_distance"),
    h3_dependency("distance_damage"),
    Float("unknown_1", VISIBLE=False),
    Float("unknown_2", VISIBLE=False),
    Float("unknown_3", VISIBLE=False),
    VISIBLE=False,
    ENDIAN=">", SIZE=120
    )


matg_material_water_drag_propertie = Struct("water_drag_propertie",
    Array("unknown_array", SUB_STRUCT=Float("unknown"), SIZE=9),
    Pad(4),
    VISIBLE=False,
    ENDIAN=">", SIZE=40
    )


matg_material_water_interaction = Struct("water_interaction",
    h3_string_id("surface_name"),
    h3_string_id("submerged_name"),
    SInt16("surface_index"),
    SInt16("submerged_index"),
    ENDIAN=">", SIZE=12
    )


matg_material = Struct("material",
    h3_string_id("name"),
    h3_string_id("parent_name"),
    SInt16("parent_index"),
    Bool16("flags", *unknown_flags_16),
    h3_string_id("general_armor"),
    h3_string_id("specific_armor"),
    BytesRaw("unknown", SIZE=4, VISIBLE=False),
    Float("friction"),
    Float("restitution"),
    Float("density"),
    h3_reflexive("water_drag_properties", matg_material_water_drag_propertie),
    h3_dependency("breakable_surface"),
    h3_dependency("sound_sweetener_small"),
    h3_dependency("sound_sweetener_medium"),
    h3_dependency("sound_sweetener_large"),
    h3_dependency("sound_sweetener_rolling"),
    h3_dependency("sound_sweetener_grinding"),
    h3_dependency("sound_sweetener_melee_small"),
    h3_dependency("sound_sweetener_melee_medium"),
    h3_dependency("sound_sweetener_melee_large"),
    h3_dependency("effect_sweetener_small"),
    h3_dependency("effect_sweetener_medium"),
    h3_dependency("effect_sweetener_large"),
    h3_dependency("effect_sweetener_rolling"),
    h3_dependency("effect_sweetener_grinding"),
    h3_dependency("effect_sweetener_melee"),
    h3_dependency("water_ripple_small"),
    h3_dependency("water_ripple_medium"),
    h3_dependency("water_ripple_large"),
    Bool32("sweetener_inheritance_flags",
        "sound_small",
        "sound_medium",
        "sound_large",
        "sound_rolling",
        "sound_grinding",
        "sound_melee_small",
        "sound_melee_medium",
        "sound_melee_large",
        "effect_small",
        "effect_medium",
        "effect_large",
        "effect_rolling",
        "effect_grinding",
        "effect_melee",
        "water_ripple_small",
        "water_ripple_medium",
        "water_ripple_large",
        ),
    h3_dependency("material_effects"),
    h3_reflexive("water_interaction", matg_material_water_interaction),
    ENDIAN=">", SIZE=368
    )


matg_player_color = Struct("player_color",
    color_rgb_float("color"),
    ENDIAN=">", SIZE=12
    )


matg_cinematic_anchor = Struct("cinematic_anchor",
    h3_dependency("cinematic_anchor"),
    BytesRaw("unknown", SIZE=4, VISIBLE=False),
    ENDIAN=">", SIZE=20
    )


matg_metagame_global_medal = QStruct("medal",
    Float("multiplier"),
    ENDIAN=">", SIZE=4
    )


matg_metagame_global_difficulty = QStruct("difficulty",
    Float("multiplier"),
    ENDIAN=">", SIZE=4
    )


matg_metagame_global_primary_skull = QStruct("primary_skull",
    Float("multiplier"),
    ENDIAN=">", SIZE=4
    )


matg_metagame_global_secondary_skull = QStruct("secondary_skull",
    Float("multiplier"),
    ENDIAN=">", SIZE=4
    )


matg_metagame_global = Struct("metagame_global",
    h3_reflexive("medals", matg_metagame_global_medal),
    h3_reflexive("difficulty", matg_metagame_global_difficulty),
    h3_reflexive("primary_skulls", matg_metagame_global_primary_skull),
    h3_reflexive("secondary_skulls", matg_metagame_global_secondary_skull),
    SInt32("unknown_0"),
    SInt32("death_penalty"),
    SInt32("betrayal_penalty"),
    SInt32("unknown_1", VISIBLE=False),
    Float("multikill_window"),
    Float("emp_window"),
    ENDIAN=">", SIZE=72
    )


matg_body = Struct("tagdata",
    BytesRaw("unknown_0", SIZE=172, VISIBLE=False),
    SEnum32("language", *ugh__language_language),
    h3_reflexive("havok_cleanup_resources", matg_havok_cleanup_resource),
    h3_reflexive("sound_globals", matg_sound_global),
    h3_reflexive("ai_globals", matg_ai_global),
    h3_reflexive("damage_table", matg_damage_table),
    BytesRaw("unknown_1", SIZE=12, VISIBLE=False),
    h3_reflexive("sounds", matg_sound),
    h3_reflexive("camera", matg_camera),
    h3_reflexive("player_control", matg_player_control),
    h3_reflexive("difficulty", matg_difficulty),
    h3_reflexive("grenades", matg_grenade),
    BytesRaw("unknown_2", SIZE=12, VISIBLE=False),
    h3_reflexive("interface_tags", matg_interface_tag),
    BytesRaw("unknown_3", SIZE=24, VISIBLE=False),
    h3_reflexive("player_information", matg_player_information),
    h3_reflexive("player_representation", matg_player_representation),
    h3_reflexive("falling_damage", matg_falling_damage),
    h3_reflexive("materials", matg_material),
    h3_reflexive("player_colors", matg_player_color),
    h3_dependency("multiplayer_globals"),
    h3_reflexive("cinematic_anchors", matg_cinematic_anchor),
    h3_reflexive("metagame_globals", matg_metagame_global),
    UInt32("unknown_4", VISIBLE=False),
    UInt32("unknown_5", VISIBLE=False),
    UInt32("string_count_0"),
    UInt32("locale_table_size_0"),
    UInt32("locale_index_table_offset_0"),
    UInt32("locale_data_index_offset_0"),
    BytesRaw("index_table_hash_0", SIZE=20),
    BytesRaw("string_data_hash_0", SIZE=20),
    UInt32("unknown_6", VISIBLE=False),
    UInt32("unknown_7", VISIBLE=False),
    UInt32("unknown_8", VISIBLE=False),
    UInt32("string_count_1"),
    UInt32("locale_table_size_1"),
    UInt32("locale_index_table_offset_1"),
    UInt32("locale_data_index_offset_1"),
    BytesRaw("index_table_hash_1", SIZE=20),
    BytesRaw("string_data_hash_1", SIZE=20),
    UInt32("unknown_9", VISIBLE=False),
    UInt32("unknown_10", VISIBLE=False),
    UInt32("unknown_11", VISIBLE=False),
    UInt32("string_count_2"),
    UInt32("locale_table_size_2"),
    UInt32("locale_index_table_offset_2"),
    UInt32("locale_data_index_offset_2"),
    BytesRaw("index_table_hash_2", SIZE=20),
    BytesRaw("string_data_hash_2", SIZE=20),
    UInt32("unknown_12", VISIBLE=False),
    UInt32("unknown_13", VISIBLE=False),
    UInt32("unknown_14", VISIBLE=False),
    UInt32("string_count_3"),
    UInt32("locale_table_size_3"),
    UInt32("locale_index_table_offset_3"),
    UInt32("locale_data_index_offset_3"),
    BytesRaw("index_table_hash_3", SIZE=20),
    BytesRaw("string_data_hash_3", SIZE=20),
    UInt32("unknown_15", VISIBLE=False),
    UInt32("unknown_16", VISIBLE=False),
    UInt32("unknown_17", VISIBLE=False),
    UInt32("string_count_4"),
    UInt32("locale_table_size_4"),
    UInt32("locale_index_table_offset_4"),
    UInt32("locale_data_index_offset_4"),
    BytesRaw("index_table_hash_4", SIZE=20),
    BytesRaw("string_data_hash_4", SIZE=20),
    UInt32("unknown_18", VISIBLE=False),
    UInt32("unknown_19", VISIBLE=False),
    UInt32("unknown_20", VISIBLE=False),
    UInt32("string_count_5"),
    UInt32("locale_table_size_5"),
    UInt32("locale_index_table_offset_5"),
    UInt32("locale_data_index_offset_5"),
    BytesRaw("index_table_hash_5", SIZE=20),
    BytesRaw("string_data_hash_5", SIZE=20),
    UInt32("unknown_21", VISIBLE=False),
    UInt32("unknown_22", VISIBLE=False),
    UInt32("unknown_23", VISIBLE=False),
    UInt32("string_count_6"),
    UInt32("locale_table_size_6"),
    UInt32("locale_index_table_offset_6"),
    UInt32("locale_data_index_offset_6"),
    BytesRaw("index_table_hash_6", SIZE=20),
    BytesRaw("string_data_hash_6", SIZE=20),
    UInt32("unknown_24", VISIBLE=False),
    UInt32("unknown_25", VISIBLE=False),
    UInt32("unknown_26", VISIBLE=False),
    UInt32("string_count_7"),
    UInt32("locale_table_size_7"),
    UInt32("locale_index_table_offset_7"),
    UInt32("locale_data_index_offset_7"),
    BytesRaw("index_table_hash_7", SIZE=20),
    BytesRaw("string_data_hash_7", SIZE=20),
    UInt32("unknown_27", VISIBLE=False),
    UInt32("unknown_28", VISIBLE=False),
    UInt32("unknown_29", VISIBLE=False),
    UInt32("string_count_8"),
    UInt32("locale_table_size_8"),
    UInt32("locale_index_table_offset_8"),
    UInt32("locale_data_index_offset_8"),
    BytesRaw("index_table_hash_8", SIZE=20),
    BytesRaw("string_data_hash_8", SIZE=20),
    UInt32("unknown_30", VISIBLE=False),
    UInt32("unknown_31", VISIBLE=False),
    UInt32("unknown_32", VISIBLE=False),
    UInt32("string_count_9"),
    UInt32("locale_table_size_9"),
    UInt32("locale_index_table_offset_9"),
    UInt32("locale_data_index_offset_9"),
    BytesRaw("index_table_hash_9", SIZE=20),
    BytesRaw("string_data_hash_9", SIZE=20),
    UInt32("unknown_33", VISIBLE=False),
    UInt32("unknown_34", VISIBLE=False),
    UInt32("unknown_35", VISIBLE=False),
    UInt32("string_count_10"),
    UInt32("locale_table_size_10"),
    UInt32("locale_index_table_offset_10"),
    UInt32("locale_data_index_offset_10"),
    BytesRaw("index_table_hash_10", SIZE=20),
    BytesRaw("string_data_hash_10", SIZE=20),
    UInt32("unknown_36", VISIBLE=False),
    UInt32("unknown_37", VISIBLE=False),
    UInt32("unknown_38", VISIBLE=False),
    UInt32("string_count_11"),
    UInt32("locale_table_size_11"),
    UInt32("locale_index_table_offset_11"),
    UInt32("locale_data_index_offset_11"),
    BytesRaw("index_table_hash_11", SIZE=20),
    BytesRaw("string_data_hash_11", SIZE=20),
    UInt32("unknown_39", VISIBLE=False),
    h3_dependency("rasterizer_globals"),
    h3_dependency("default_camera_effect"),
    h3_dependency("default_wind"),
    h3_dependency("default_damage_effect"),
    h3_dependency("default_collision_damage"),
    h3_string_id("unknown_material"),
    SInt16("unknown_global_material_index"),
    SInt16("unknown_40", VISIBLE=False),
    h3_dependency("effect_globals"),
    BytesRaw("unknown_41", SIZE=172, VISIBLE=False),
    ENDIAN=">", SIZE=1536
    )


def get():
    return matg_def

matg_def = TagDef("matg",
    h3_blam_header('matg'),
    matg_body,

    ext=".%s" % h3_tag_class_fcc_to_ext["matg"], endian=">", tag_cls=H3Tag
    )



#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

__all__ = (
    'generate_fake_nodes',
    'edge_loop_to_strippable_tris',
    'edge_loop_to_fannable_tris',
    'edge_loop_to_tris',
    )

from .node import JmsNode
from .triangle import JmsTriangle


def generate_fake_nodes(node_count):
    nodes = []
    if node_count <= 0:
        return nodes

    nodes.append(JmsNode("fake_node0", 1, -1))
    for i in range(1, node_count):
        nodes.append(JmsNode("fake_node%s" % i, -1, i + 1))

    nodes[-1].first_child = -1
    nodes[-1].sibling_index = -1
    JmsNode.setup_node_hierarchy(nodes)
    return nodes


def edge_loop_to_strippable_tris(edge_loop, region=0, mat_id=0):
    tris = [None] * (len(edge_loop) - 2)
    vert_ct = len(edge_loop)

    even_face_ct = (vert_ct - 1) // 2
    odd_face_ct = (vert_ct - 2) // 2

    # make the even faces
    v0 = edge_loop[0]
    for i in range(even_face_ct):
        v1 = edge_loop[i + 1]
        v2 = edge_loop[vert_ct - 1 - i]
        tris[i << 1] = JmsTriangle(region, mat_id, v0, v1, v2)

        v0 = v2

    # make the odd faces
    v0 = edge_loop[1]
    for i in range(odd_face_ct):
        v1 = edge_loop[i + 2]
        v2 = edge_loop[vert_ct - 1 - i]
        tris[(i << 1) + 1] = JmsTriangle(region, mat_id, v0, v1, v2)

        v0 = v1

    return tris


def edge_loop_to_fannable_tris(edge_loop, region=0, mat_id=0):
    vert_index_count = len(edge_loop)
    v0 = edge_loop[0]
    return [JmsTriangle(region, mat_id, v0,
                        edge_loop[((i + 1) % vert_index_count)],
                        edge_loop[((i + 2) % vert_index_count)])
            for i in range(len(edge_loop) - 2)]


def edge_loop_to_tris(edge_loop_or_vert_index_count, region=0, mat_id=0,
                      base=0, make_fan=False):
    if isinstance(edge_loop_or_vert_index_count, int):
        edge_loop = list(range(base, base + edge_loop_or_vert_index_count))
    else:
        edge_loop = edge_loop_or_vert_index_count

    if make_fan:
        return edge_loop_to_fannable_tris(edge_loop, region, mat_id)

    return edge_loop_to_strippable_tris(edge_loop, region, mat_id)



#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

from ...common_descs import *
from .objs.tag import HekTag
from supyr_struct.defs.tag_def import TagDef

devc_body = Struct("tagdata",
    SEnum16("device_type",
        "mouse_and_keyboard",
        "joysticks_joypads_etc",
        "full_profile_definition",
        ),
    Bool16("flags",
        "unused",
        ),
    rawdata_ref("device_id", max_size=16),
    rawdata_ref("profile", max_size=41984),
    SIZE=44,
    )

def get():
    return devc_def

devc_def = TagDef("devc",
    blam_header('devc'),
    devc_body,

    ext=".input_device_defaults", endian=">", tag_cls=HekTag
    )



#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

from reclaimer.hek.defs.objs.tag import HekTag

class CollTag(HekTag):

    def calc_internal_data(self):
        HekTag.calc_internal_data(self)

        shield = self.data.tagdata.shield
        shield.shield_recharge_rate = 0
        if shield.recharge_time:
            shield.shield_recharge_rate = 1 / shield.recharge_time
        shield.shield_recharge_rate /= 30



#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

############# Credits and version info #############
# Definition generated from Assembly XML tag def
#	 Date generated: 2018/12/03  04:56
#
# revision: 1		author: Assembly
# 	Generated plugin from scratch.
# revision: 2		author: Lord Zedd
# 	Updated and standardized.
# revision: 3		author: Moses_of_Egypt
# 	Cleaned up and converted to SuPyr definition
#
####################################################

from ..common_descs import *
from .objs.tag import *
from supyr_struct.defs.tag_def import TagDef


uise_body = Struct("tagdata",
    h3_dependency("error"),
    h3_dependency("vertical_navigation"),
    h3_dependency("horizontal_navigation"),
    h3_dependency("a_button"),
    h3_dependency("b_button"),
    h3_dependency("x_button"),
    h3_dependency("y_button"),
    h3_dependency("start_button"),
    h3_dependency("back_button"),
    h3_dependency("left_bumper"),
    h3_dependency("right_bumper"),
    h3_dependency("left_trigger"),
    h3_dependency("right_trigger"),
    h3_dependency("timer_sound"),
    h3_dependency("timer_sound_zero"),
    h3_dependency("alt_timer_sound"),
    h3_dependency("second_alt_timer_sound"),
    h3_dependency("matchmaking_advance_sound"),
    h3_dependency("rank_up"),
    h3_dependency("matchmaking_party_up_sound"),
    ENDIAN=">", SIZE=320
    )


def get():
    return uise_def

uise_def = TagDef("uise",
    h3_blam_header('uise'),
    uise_body,

    ext=".%s" % h3_tag_class_fcc_to_ext["uise"], endian=">", tag_cls=H3Tag
    )



#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

############# Credits and version info #############
# Definition generated from Assembly XML tag def
#	 Date generated: 2018/12/03  04:56
#
# revision: 1		author: -DeToX-
# 	Mapped plugin structure a new.
# revision: 2		author: Lord Zedd
# 	thx h2
# revision: 3		author: Moses_of_Egypt
# 	Cleaned up and converted to SuPyr definition
#
####################################################

from ..common_descs import *
from .objs.tag import *
from supyr_struct.defs.tag_def import TagDef


mdlg_line_variant = Struct("variant",
    h3_string_id("variant_designation"),
    h3_dependency("sound"),
    h3_string_id("sound_effect"),
    VISIBLE=False,
    ENDIAN=">", SIZE=24
    )


mdlg_line = Struct("line",
    h3_string_id("name"),
    h3_reflexive("variants", mdlg_line_variant),
    h3_string_id("default_sound_effect"),
    ENDIAN=">", SIZE=20
    )


mdlg_body = Struct("tagdata",
    h3_reflexive("lines", mdlg_line),
    ENDIAN=">", SIZE=12
    )


def get():
    return mdlg_def

mdlg_def = TagDef("mdlg",
    h3_blam_header('mdlg'),
    mdlg_body,

    ext=".%s" % h3_tag_class_fcc_to_ext["mdlg"], endian=">", tag_cls=H3Tag
    )



#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

from ...common_descs import *
from .objs.antr import AntrTag
from supyr_struct.defs.tag_def import TagDef

frame_info_dxdy_node = QStruct("frame_info_node",
    Float("dx"),
    Float("dy"), ORIENT='h'
    )

frame_info_dxdydyaw_node = QStruct("frame_info_node",
    Float("dx"),
    Float("dy"),
    Float("dyaw"), ORIENT='h'
    )

frame_info_dxdydzdyaw_node = QStruct("frame_info_node",
    Float("dx"),
    Float("dy"),
    Float("dz"),
    Float("dyaw"), ORIENT='h'
    )

default_node = Struct("default_node",
    # each of these structs exists ONLY if the corrosponding flag
    # for that node it NOT set in the animation it is located in.
    QStruct("rotation",
        SInt16("i", UNIT_SCALE=1/32767),
        SInt16("j", UNIT_SCALE=1/32767),
        SInt16("k", UNIT_SCALE=1/32767),
        SInt16("w", UNIT_SCALE=1/32767),
        ORIENT="h"
        ),
    QStruct("translation", INCLUDE=xyz_float),
    Float("scale"),
    SIZE=24
    )


dyn_anim_path = "tagdata.animations.STEPTREE[DYN_I].name"

object_desc = Struct("object",
    dyn_senum16("animation", DYN_NAME_PATH=dyn_anim_path),
    SEnum16("function",
        "A_out",
        "B_out",
        "C_out",
        "D_out"
        ),
    SEnum16("function_controls",
        "frame",
        "scale",
        ),
    SIZE=20,
    )

anim_enum_desc = QStruct("animation",
    dyn_senum16("animation", DYN_NAME_PATH=dyn_anim_path)
    )

ik_point_desc = Struct("ik_point",
    ascii_str32("marker"),
    ascii_str32("attach_to_marker"),
    SIZE=64,
    )

weapon_types_desc = Struct("weapon_types",
    ascii_str32("label"),
    Pad(16),
    reflexive("animations", anim_enum_desc, 10,
        *unit_weapon_type_animation_names
        ),
    SIZE=60,
    )

unit_weapon_desc = Struct("weapon",
    ascii_str32("name"),
    ascii_str32("grip_marker"),
    ascii_str32("hand_marker"),
    #Aiming screen bounds

    #pitch and yaw are saved in radians.
    float_rad("right_yaw_per_frame"),
    float_rad("left_yaw_per_frame"),
    SInt16("right_frame_count"),
    SInt16("left_frame_count"),

    float_rad("down_pitch_per_frame"),
    float_rad("up_pitch_per_frame"),
    SInt16("down_frame_count"),
    SInt16("up_frame_count"),

    Pad(32),
    reflexive("animations", anim_enum_desc, 55,
        *unit_weapon_animation_names
        ),
    reflexive("ik_points", ik_point_desc, 4, DYN_NAME_PATH=".marker"),
    reflexive("weapon_types", weapon_types_desc, 10, DYN_NAME_PATH=".label"),
    SIZE=188,
    )

unit_desc = Struct("unit",
    ascii_str32("label"),
    #pitch and yaw are saved in radians.

    #Looking screen bounds
    float_rad("right_yaw_per_frame"),
    float_rad("left_yaw_per_frame"),
    SInt16("right_frame_count"),
    SInt16("left_frame_count"),

    float_rad("down_pitch_per_frame"),
    float_rad("up_pitch_per_frame"),
    SInt16("down_frame_count"),
    SInt16("up_frame_count"),

    Pad(8),
    reflexive("animations", anim_enum_desc, 30,
        *unit_animation_names
        ),
    reflexive("ik_points", ik_point_desc, 4, DYN_NAME_PATH=".marker"),
    reflexive("weapons", unit_weapon_desc, 16, DYN_NAME_PATH=".name"),
    SIZE=100,
    )

weapon_desc = Struct("weapon",
    Pad(16),
    reflexive("animations", anim_enum_desc, 11,
        *weapon_animation_names
        ),
    SIZE=28,
    )

suspension_desc = QStruct("suspension_animation",
    SInt16("mass_point_index"),
    dyn_senum16("animation", DYN_NAME_PATH=dyn_anim_path),
    Float("full_extension_ground_depth"),
    Float("full_compression_ground_depth"),
    SIZE=20,
    )

vehicle_desc = Struct("vehicle",
    #pitch and yaw are saved in radians.

    #Steering screen bounds
    float_rad("right_yaw_per_frame"),
    float_rad("left_yaw_per_frame"),
    SInt16("right_frame_count"),
    SInt16("left_frame_count"),

    float_rad("down_pitch_per_frame"),
    float_rad("up_pitch_per_frame"),
    SInt16("down_frame_count"),
    SInt16("up_frame_count"),

    Pad(68),
    reflexive("animations", anim_enum_desc, 8,
        *vehicle_animation_names
        ),
    reflexive("suspension_animations", suspension_desc, 8),
    SIZE=116,
    )

device_desc = Struct("device",
    Pad(84),
    reflexive("animations", anim_enum_desc, 2,
        *device_animation_names
        ),
    SIZE=96,
    )

fp_animation_desc = Struct("fp_animation",
    Pad(16),
    reflexive("animations", anim_enum_desc, 28,
        *fp_animation_names
        ),
    SIZE=28,
    )

sound_reference_desc = Struct("sound_reference",
    dependency('sound', "snd!"),
    SIZE=20,
    )

nodes_desc = Struct("node",
    ascii_str32("name"),
    dyn_senum16("next_sibling_node_index", DYN_NAME_PATH="..[DYN_I].name"),
    dyn_senum16("first_child_node_index", DYN_NAME_PATH="..[DYN_I].name"),
    dyn_senum16("parent_node_index", DYN_NAME_PATH="..[DYN_I].name"),
    Pad(2),
    Bool32("node_joint_flags",
        "ball_socket",
        "hinge",
        "no_movement",
        ),
    QStruct("base_vector", INCLUDE=ijk_float),
    float_rad("vector_range"),
    Pad(4),
    SIZE=64,
    )

animation_desc = Struct("animation",
    ascii_str32("name"),
    SEnum16("type", *anim_types),
    SInt16("frame_count"),
    SInt16("frame_size"),
    SEnum16("frame_info_type", *anim_frame_info_types),
    SInt32("node_list_checksum"),
    SInt16("node_count"),
    SInt16("loop_frame_index"),

    Float("weight"),
    SInt16("key_frame_index"),
    SInt16("second_key_frame_index"),

    dyn_senum16("next_animation",
        DYN_NAME_PATH="..[DYN_I].name"),
    Bool16("flags",
        "compressed_data",
        "world_relative",
        { NAME:"pal", GUI_NAME:"25Hz(PAL)" },
        ),
    dyn_senum16("sound",
        DYN_NAME_PATH="tagdata.sound_references." +
        "sound_references_array[DYN_I].sound.filepath"),
    SInt16("sound_frame_index"),
    SInt8("left_foot_frame_index"),
    SInt8("right_foot_frame_index"),
    FlSInt16("first_permutation_index", VISIBLE=False,
        TOOLTIP="The index of the first animation in the permutation chain."),
    FlFloat("chance_to_play", VISIBLE=False,
        MIN=0.0, MAX=1.0, SIDETIP="[0,1]",
        TOOLTIP=("Seems to be the chance range to select this permutation.\n"
                 "Random number in the range [0,1] is rolled. The permutation\n"
                 "chain is looped until the number is higher than or equal\n"
                 "to that permutations chance to play. This chance to play\n"
                 "is likely influenced by the animations 'weight' field.\n"
                 "All permutation chains should have the last one end with\n"
                 "a chance to play of 1.0")),

    rawdata_ref("frame_info", max_size=32768),

    # each of the bits in these flags determines whether
    # or not the frame data stores info for each nodes
    # translation, rotation, and scale.
    # This info was discovered by looking at TheGhost's
    # animation importer script, so thank him for that.
    UInt32("trans_flags0", EDITABLE=False),
    UInt32("trans_flags1", EDITABLE=False),
    Pad(8),
    UInt32("rot_flags0", EDITABLE=False),
    UInt32("rot_flags1", EDITABLE=False),
    Pad(8),
    UInt32("scale_flags0", EDITABLE=False),
    UInt32("scale_flags1", EDITABLE=False),
    Pad(4),
    SInt32("offset_to_compressed_data", EDITABLE=False),
    rawdata_ref("default_data", max_size=16384),
    rawdata_ref("frame_data", max_size=1048576),
    SIZE=180,
    )

antr_body = Struct("tagdata",
    reflexive("objects",  object_desc, 4),
    reflexive("units",    unit_desc, 32, DYN_NAME_PATH=".label"),
    reflexive("weapons",  weapon_desc, 1),
    reflexive("vehicles", vehicle_desc, 1),
    reflexive("devices",  device_desc, 1),
    reflexive("unit_damages", anim_enum_desc, 176,
        *unit_damage_animation_names
        ),
    reflexive("fp_animations", fp_animation_desc, 1),
    #i have no idea why they decided to cap it at 257 instead of 256....
    reflexive("sound_references", sound_reference_desc, 257,
        DYN_NAME_PATH=".sound.filepath"),
    Float("limp_body_node_radius"),
    Bool16("flags",
        "compress_all_animations",
        "force_idle_compression",
        ),
    Pad(2),
    reflexive("nodes", nodes_desc, 64, DYN_NAME_PATH=".name"),
    reflexive("animations", animation_desc, 256, DYN_NAME_PATH=".name"),
    SIZE=128,
    )


def get():
    return antr_def

antr_def = TagDef("antr",
    blam_header('antr', 4),
    antr_body,

    ext=".model_animations", endian=">", tag_cls=AntrTag
    )



#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

import hashlib, hmac

from supyr_struct.buffer import BytearrayBuffer
from supyr_struct.tag import Tag

DEFAULT_XBOX_KEY = (b'\x5C\x07\x33\xAE\x04\x01\xF7\xE8'+
                    b'\xBA\x79\x93\xFD\xCD\x2F\x1F\xE0')

class XboxSaveTag(Tag):
    sigkey  = None
    authkey = None
    xboxkey = DEFAULT_XBOX_KEY

    data_start = 20
    data_end   = 0

    def __init__(self, **kwargs):
        self.sigkey  = kwargs.get('sigkey',  self.sigkey)
        self.authkey = kwargs.get('authkey', self.authkey)
        self.xboxkey = kwargs.get('xboxkey', self.xboxkey)
        self.data_start = kwargs.get('data_start', self.data_start)
        self.data_end = kwargs.get('data_end', self.data_end)
        self.sigkey   = kwargs.get('sigkey',   self.sigkey)

        Tag.__init__(self, **kwargs)

    def calc_authkey(self, sigkey=None):
        if sigkey is None:
            sigkey = self.sigkey
        authkey = hmac.new(self.xboxkey, sigkey[:16], hashlib.sha1)
        authkey = authkey.digest()[:16]
        if self.authkey is None:
            self.authkey = authkey
        return authkey

    def calc_hmac_sig(self, rawdata, authkey=None):
        authkey = self.authkey if authkey is None else authkey
        if authkey is None:
            authkey = self.calc_authkey(self.sigkey)
        return hmac.new(authkey, rawdata, hashlib.sha1).digest()

    def xbox_sign(self, rawdata=None, authkey=None):
        if rawdata is None:
            rawdata = self.data.serialize(buffer=BytearrayBuffer())
            if self.data_end != 0:
                rawdata = rawdata[self.data_start:self.data_end]
            else:
                rawdata = rawdata[self.data_start:]
        hmac_sig = self.calc_hmac_sig(rawdata, authkey)
        self.data.hmac_sig = hmac_sig

    def serialize(self, **kwargs):
        self.xbox_sign()
        return Tag.serialize(self, **kwargs)



#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

from reclaimer.common_descs import *

# ###########################################################################
# The order of element in all the enumerators is important(DONT SHUFFLE THEM)
# ###########################################################################

#Shared Enumerator options
materials_list = (
    "unknown1",
    "unknown2",
    "unknown3",
    "unknown4",
    "unknown5",
    "unknown6",
    "unknown7",
    "unknown8",
    "unknown9",
    "unknown10",
    "unknown11",
    "unknown12",
    "unknown13",
    "unknown14",
    "unknown15",
    "unknown16",
    "unknown17",
    "unknown18",
    "unknown19",
    "unknown20",
    "unknown21",
    "unknown22",
    "unknown23",
    "unknown24",
    "unknown25",
    "unknown26",
    "unknown27",
    "unknown28",
    "unknown29",
    "unknown30",
    "unknown31",
    "unknown32",
    "unknown33",
    "unknown34",
    "unknown35",
    )
actor_types = (
    "unknown1",
    "unknown2",
    "unknown3",
    "unknown4",
    "unknown5",
    "unknown6",
    "unknown7",
    "unknown8",
    "unknown9",
    "unknown10",
    "unknown11",
    "unknown12",
    "unknown13",
    "unknown14",
    "unknown15",
    "unknown16",
    )
damage_category = (
    "unknown1",
    "unknown2",
    "unknown3",
    "unknown4",
    "unknown5",
    "unknown6",
    "unknown7",
    "unknown8",
    "unknown9",
    "unknown10",
    "unknown11",
    "unknown12",
    "unknown13",
    )
grenade_types = (
    'unknown1',
    'unknown2',
    'unknown3',
    'unknown4',
    )

#Tag class specific enumerators
unit_teams = (
    "none",
    "player",
    "unknown1",
    "unknown2",
    "unknown3",
    "unknown4",
    "unknown5",
    "unknown6",
    "unknown7",
    "unknown8",
    )
vehicle_types = (
    "unknown1",
    "unknown2",
    "unknown3",
    "unknown4",
    "unknown5",
    "unknown6",
    "unknown7",
    )
weapon_types = (
    "unknown1",
    "unknown2",
    "unknown3",
    "unknown4",
    "unknown5",
    )

damage_modifiers = QStruct("damage_modifiers",
    *(float_zero_to_inf(material_name) for material_name in materials_list)
    )


def tag_class_stubbs(*args, **kwargs):
    '''
    A macro for creating a tag_class enum desc with the
    enumerations set to the provided tag_class fcc's.
    '''
    kwargs["class_mapping"] = tag_class_fcc_to_ext_stubbs
    return tag_class(*args, **kwargs)


def dependency_stubbs(name='tag_ref', valid_ids=None, **kwargs):
    '''This function serves to macro the creation of a tag dependency'''
    if isinstance(valid_ids, tuple):
        valid_ids = tag_class_stubbs(*valid_ids)
    elif isinstance(valid_ids, str):
        valid_ids = tag_class_stubbs(valid_ids)
    elif valid_ids is None:
        valid_ids = stubbs_valid_tags

    return TagRef(name,
        valid_ids,
        INCLUDE=tag_ref_struct,
        STEPTREE=StrTagRef(
            "filepath", SIZE=tag_ref_str_size, GUI_NAME="", MAX=234),
        **kwargs
        )


def blam_header_stubbs(tagid, version=1):
    '''This function serves to macro the creation of a tag header'''
    header_desc= dict(tag_header_stubbs)
    header_desc[1] = dict(header_desc[1])
    header_desc[5] = dict(header_desc[5])
    header_desc[1][DEFAULT] = tagid
    header_desc[5][DEFAULT] = version
    return header_desc


stubbs_valid_tags = tag_class_stubbs(*tag_class_fcc_to_ext_stubbs.keys())


# Descriptors
tag_header_stubbs = Struct("blam_header",
    Pad(36),
    UEnum32("tag_class",
        GUI_NAME="tag_class", INCLUDE=stubbs_valid_tags, EDITABLE=False
        ),
    UInt32("checksum", DEFAULT=0x4D6F7A7A, EDITABLE=False),
    UInt32("header_size",  DEFAULT=64, EDITABLE=False),
    BBool64("flags",
        "edited_with_mozz",
        EDITABLE=False
        ),
    UInt16("version", DEFAULT=1, EDITABLE=False),
    UInt8("integrity0", DEFAULT=0, EDITABLE=False),
    UInt8("integrity1", DEFAULT=255, EDITABLE=False),
    UEnum32("engine_id",
        ("halo_1", 'blam'),
        DEFAULT='blam', EDITABLE=False
        ),
    VISIBLE=VISIBILITY_METADATA, SIZE=64, ENDIAN=">"  # KEEP THE ENDIAN SPECIFICATION
    )



#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

from .obje import *
from .item import *
from .objs.obje import ObjeTag
from supyr_struct.defs.tag_def import TagDef

# replace the object_type enum one that uses
# the correct default value for this object
obje_attrs = dict(obje_attrs)
obje_attrs[0] = dict(obje_attrs[0], DEFAULT=3)

eqip_attrs = Struct("eqip_attrs",
    SEnum16('powerup_type',
        'none',
        'double_speed',
        'overshield',
        'active_camo',
        'full_spectrum_vision',
        'health',
        'grenade',
        ),
    SEnum16('grenade_type', *grenade_types),
    float_sec('powerup_time'),
    dependency('pickup_sound', "snd!"),

    SIZE=168
    )

eqip_body = Struct("tagdata",
    obje_attrs,
    item_attrs,
    eqip_attrs,

    SIZE=944,
    )


def get():
    return eqip_def

eqip_def = TagDef("eqip",
    blam_header('eqip', 2),
    eqip_body,

    ext=".equipment", endian=">", tag_cls=ObjeTag
    )



#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

from ...hek.defs.pctl import *

particle_state = dict(particle_state)
particle_type  = dict(particle_type)
pctl_body      = dict(pctl_body)

particle_state[19] = reflexive(
    "shader_extensions",
    Struct("shader_extension", INCLUDE=os_shader_extension),
    1)
particle_type[12] = reflexive(
    "particle_states", particle_state, 8, DYN_NAME_PATH='.name')
pctl_body[5] = reflexive(
    "particle_types", particle_type, 4, DYN_NAME_PATH='.name')


def get():
    return pctl_def

pctl_def = TagDef("pctl",
    blam_header("pctl", 4),
    pctl_body,

    ext=".particle_system", endian=">", tag_cls=HekTag,
    )



#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

from ...common_descs import *
from .objs.tag import HekTag
from supyr_struct.defs.tag_def import TagDef

decal_comment = """COMPOUND DECALS:
A 'compound decal' is a chain of decals which are instantiated simultaneously.
Compound decals are created by choosing a <next_decal_in_chain> below.
NOTE: Do not attempt to create a circularly linked decal chain, i.e. A->B->C->A!
Also, do not reference a decal from an effect if it is not the 'head' of the chain;
for example an effect should not instantiate decal B if the chain was A->B->C.
Compound decals can have seperate bitmaps, seperate framebuffer blend functions,
and can be drawn in seperate layers. In addition, each decal in the chain can either
inherit its parent's <radius>, rotation, <color>, <fade>, and <sequence> -
or it can randomly choose its own. This behavior is controlled by the
'geometry_inherited_by_next_decal_in_chain' flag, below.

DECAL TYPING AND LAYERING:
The decal <type> (or layer) determines the drawing order of the decal with respect
to the rest of the environment. Decals in the primary layer are drawn after the
environment diffuse texture, hence they affect the already-lit texture of the surface.
Decals in the secondary layer are drawn immediately after decals in the primary layer,
so they 'cover up' the primary decals. Decals in the 'light' layer are drawn before
the environment diffuse texture, hence they affect the accumulated diffuse light and
only indirectly affect the lit texture."""

deca_body = Struct("tagdata",
    #Decal Properties
    Bool16("flags",
        "geometry_inherited_by_next_decal_in_chain",
        "interpolate_color_in_hsv",
        "more_colors",
        "no_random_rotation",
        "water_effect",
        "SAPIEN_snap_to_axis",
        "SAPIEN_incremental_counter",
        "animation_loop",
        "preserve_aspect",
        COMMENT=decal_comment
        ),
    SEnum16("type",
        "scratch",
        "splatter",
        "burn",
        "painted_sign",
        ),
    SEnum16("layer",
        "primary",
        "secondary",
        "light",
        "alpha_tested",
        "water"
        ),
    Pad(2),
    dependency("next_decal_in_chain", "deca"),
    from_to_wu("radius"),  # world units
    Pad(12),

    Struct("color",
        from_to_zero_to_one("intensity"),  # [0,1]
        Struct("lower_bounds", INCLUDE=rgb_float),
        Struct("upper_bounds", INCLUDE=rgb_float),
        Pad(12),
        ),

    #Animation
    Struct("animation",
        SInt16("loop_frame"),
        SInt16("speed", MIN=1, MAX=120,
                SIDETIP="[1,120] ticks/frame", UNIT_SCALE=per_sec_unit_scale),
        Pad(28),
        from_to_sec("lifetime"),  # seconds
        from_to_sec("decay_time"),  # seconds
        Pad(56),
        ),

    #Shader
    Struct("shader",
        SEnum16("framebuffer_blend_function", *framebuffer_blend_functions),
        Pad(22),
        dependency("shader_map", "bitm"),
        ),

    #Sprite info
    Pad(20),
    Float("maximum_sprite_extent", SIDETIP="pixels"),

    SIZE=268,
    )



def get():
    return deca_def

deca_def = TagDef("deca",
    blam_header('deca'),
    deca_body,

    ext=".decal", endian=">", tag_cls=HekTag
    )



#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

from ...os_v3_hek.defs.unit import *

region_targeting_comment="""
When a target region is defined, melee damage is directed at it until it is destroyed.
"""

unit_keyframe_action = Struct("unit_keyframe_action",
    SEnum16("keyframe",
        "primary",
        "secondary",
        "final",
        ),
    Bool16("flags",
        "eject_mounted_units",
        ),
    SEnum16("target",
        "all_seats",
        "mounted_trigger_seats",
        ),
    Pad(2),

    dependency('damage_effect', "jpt!"),
    dependency('effect', "effe"),
    ascii_str32("effect_marker"),
    SIZE=96
    )

seat_keyframe_action = Struct("seat_keyframe_action",
    SEnum16("keyframe",
        "primary",
        "secondary",
        "final",
        ),
    Bool16("flags",
        "control_powered_seat",
        ),
    SEnum16("self_seat_action",
        "none",
        "exit_current_seat",
        "enter_target_seat",
        ),
    SEnum16("target_seat_unit_action",
        "none",
        "exit_seat",
        "eject_from_seat",
        ),
    SEnum16("unit_door_action",
        "neither",
        "open",
        "close",
        ),
    Pad(2),
    SEnum16("apply_damage_to",
        "none",
        "mounted_unit",
        "mounted_unit_region",
        "seated_unit",
        ),
    Pad(2),

    ascii_str32("region_name"),
    dependency('damage_effect', "jpt!"),

    SEnum16("apply_effect_to",
        "none",
        "mounted_unit",
        "seated_unit",
        ),
    Pad(2),

    dependency('effect', "effe"),
    ascii_str32("effect_marker"),

    SIZE=152
    )

mounted_state = Struct("mounted_state",
    Bool16("flags",
        "third_person_camera"
        ),
    Pad(2),

    ascii_str32("camera_marker_name"),
    ascii_str32("camera_submerged_marker_name"),
    float_deg("pitch_autolevel"),
    from_to_deg("pitch_range"),
    reflexive("camera_tracks", camera_track, 2,
        'loose', 'tight'),
    reflexive("keyframe_actions", unit_keyframe_action, 12,
        DYN_NAME_PATH='.effect_marker'),
    SIZE=128
    )

seat_access = Struct("seat_access",
    Bool16("flags",
        "requires_target_seat_occupied",
        "enemy_access_only",
        "restrict_by_unit_sight",
        "restrict_by_mounting_unit_sight",
        "restrict_by_unit_shield",
        "restrict_by_unit_health",
        "restrict_by_ai_state",
        ),
    Pad(2),
    float_rad("unit_sight_angle"),
    ascii_str32("unit_sight_marker_name"),
    float_rad("mounting_unit_sight_angle"),
    ascii_str32("mounting_unit_sight_marker_name"),
    float_zero_to_one("unit_shield_threshold"),
    float_zero_to_one("unit_health_threshold"),
    Bool16("permitted_ai_states", *actor_states),

    SIZE=124
    )

seat_boarding = Struct("seat_boarding",
    SEnum16("boarding_type",
        "immediate",
        "delayed"
        ),
    Bool16("delay_until",
        "empty_target_seat",
        "unit_shield_threshold",
        "unit_health_threshold",
        "region_destroyed",
        ),
    SEnum16("unit_vitality_source",
        "mounted_unit",
        "seated_unit"
        ),
    Pad(2),

    float_zero_to_one("unit_shield_threshold"),
    float_zero_to_one("unit_health_threshold"),
    Pad(4),

    ascii_str32("region_name"),
    Pad(24),

    reflexive("seat_keyframe_actions", seat_keyframe_action, 12,
        DYN_NAME_PATH='.region_name'),

    SIZE=112
    )

seat_damage = Struct("seat_damage",
    Bool16("flags",
        "use_weapon_damage_melee_effect",
        "exit_after_grenade_plant",
        ),
    Struct("melee_damage",
        SEnum16("melee",
            "normal",
            "mounted_unit",
            "target_seat_unit"
            ),
        dependency("damage_effect", "jpt!"),
        ),

    Struct("grenade_damage",
        SEnum16("grenade",
            "normal",
            "disabled",
            "plant_on_mounted_unit",
            "plant_on_target_seat_unit"
            ),
        Bool16("disabled_types",
            "fragmentation_grenade",
            "plasma_grenade",
            "custom_grenade_1",
            "custom_grenade_2",
            ),
        float_zero_to_inf("detonation_time", UNIT_SCALE=sec_unit_scale),
        ascii_str32("attach_marker"),
        ),

    Struct("region_targeting",
        Bool16("flags",
            "disable_grenades_until_destroyed"
            ),
        Pad(2),

        ascii_str32("region_name"),
        dependency("damage_effect", "jpt!"),
        COMMENT=region_targeting_comment
        ),
    SIZE=136
    )

unit_extension = Struct("unit_extension",
    reflexive("mounted_states", mounted_state, 1),
    SIZE=60
    )

seat_extension = Struct("seat_extension",
    Bool16("flags",
        "triggers_mounted_state",
        "exit_on_unit_death",
        "exit_on_target_seat_empty",
        "prevent_death_when_unit_dies",
        "ignored_by_seated_ai",
        "ignored_by_mounted_ai",
        "cant_enter_seats_when_occupied",
        ),
    dyn_senum16("target_seat",
        DYN_NAME_PATH="tagdata.unit_attrs.seats.STEPTREE[DYN_I].label"),
    Pad(12),  # reflexive("unknown", unknown),
    reflexive("seat_access", seat_access, 1),
    reflexive("seat_boarding", seat_boarding, 1),
    reflexive("seat_damage", seat_damage, 1),
    SIZE=100
    )

seat = dict(seat)
unit_attrs = dict(unit_attrs)

seat[0] = Bool32("flags",
    "invisible",
    "locked",
    "driver",
    "gunner",
    "third_person_camera",
    "allows_weapons",
    "third_person_on_enter",
    "first_person_slaved_to_gun",
    "allow_vehicle_communcation_animation",
    "not_valid_without_driver",
    "allow_ai_noncombatants",
    ("allows_melee", 1<<20)
    )
seat[20] = reflexive("seat_extensions", seat_extension, 1)
unit_attrs[45] = reflexive("unit_extensions", unit_extension, 1)
unit_attrs[54] = reflexive("seats", seat, 16, DYN_NAME_PATH='.label')

unit_body = Struct('tagdata', unit_attrs)

def get():
    return unit_def

unit_def = TagDef("unit",
    blam_header('unit', 2),
    unit_body,

    ext=".unit", endian=">", tag_cls=HekTag
    )



#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

from ...common_descs import *
from ...hek.defs.objs.tag import HekTag
from supyr_struct.defs.tag_def import TagDef

shader_pass = Struct("pass",
    ascii_str32("name"),
    Bool16("flags",
        "clear_target",
        "copy_scene_to_target",
        "clear_buffer_texture",
        ),
    SEnum16("render_chain",
        "main_chain",
        "buffer_chain"
        ),
    SIZE=48,
    )

technique = Struct("entry",
    ascii_str32("name"),
    Bool16("shader_model",
        {NAME: "sm_1_0", GUI_NAME: "1.0"},
        {NAME: "sm_2_0", GUI_NAME: "2.0"},
        {NAME: "sm_3_0", GUI_NAME: "3.0"},
        ),

    Pad(18),
    reflexive("shader_pass", shader_pass, 32,
        DYN_NAME_PATH='.name'),
    SIZE=64
    )

shpp_attrs = Struct("shpp",
    Pad(24),
    rawdata_ref("shader_code_binary", max_size=32768),

    Pad(64),
    reflexive("techniques", technique, 3,
        DYN_NAME_PATH='.name'),
    reflexive("predicted_resources", predicted_resource, 1024, VISIBLE=False),
    SIZE=164
    )

shpp_body = Struct("tagdata", shpp_attrs)

def get():
    return shpp_def

shpp_def = TagDef("shpp",
    blam_header_os('shpp'),
    shpp_body,

    ext=".shader_postprocess", endian=">", tag_cls=HekTag
    )



#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

############# Credits and version info #############
# Definition generated from Assembly XML tag def
#	 Date generated: 2018/12/03  04:56
#
# revision: 1		author: Assembly
# 	Generated plugin from scratch.
# revision: 2		author: DarkShallFall
# 	Idents labled.
# revision: 3		author: Lord Zedd
# 	Basically done.
# revision: 4		author: Moses_of_Egypt
# 	Cleaned up and converted to SuPyr definition
#
####################################################

from ..common_descs import *
from .objs.tag import *
from supyr_struct.defs.tag_def import TagDef


wgan_body = Struct("tagdata",
    h3_string_id("unknown_0"),
    BytesRaw("unknown_1", SIZE=4, VISIBLE=False),
    h3_dependency("widget_color"),
    h3_dependency("widget_position"),
    h3_dependency("widget_rotation"),
    h3_dependency("widget_scale"),
    h3_dependency("widget_texture_coordinate"),
    h3_dependency("widget_sprite"),
    h3_dependency("widget_font"),
    ENDIAN=">", SIZE=120
    )


def get():
    return wgan_def

wgan_def = TagDef("wgan",
    h3_blam_header('wgan'),
    wgan_body,

    ext=".%s" % h3_tag_class_fcc_to_ext["wgan"], endian=">", tag_cls=H3Tag
    )



#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

from ...hek.defs.spla import *
from .shdr import *
from supyr_struct.defs.tag_def import TagDef

spla_body = Struct("tagdata",
    shdr_attrs,
    spla_attrs,
    SIZE=332,
    )


def get():
    return spla_def

spla_def = TagDef("spla",
    blam_header_stubbs('spla'),
    spla_body,

    ext=".shader_transparent_plasma", endian=">", tag_cls=ShdrTag
    )
